@@ -1,7 +1,7 @@
 # pylint: disable=too-many-instance-attributes, too-many-arguments
 from contextlib import suppress
 from enum import Enum
-from inspect import Signature, isawaitable, isclass, iscoroutinefunction, ismethod
+from inspect import Signature, isawaitable, isclass, ismethod
 from typing import (
     TYPE_CHECKING,
     Any,
@@ -39,10 +39,12 @@ from starlite.types import (
     AfterRequestHandler,
     BeforeRequestHandler,
     CacheKeyBuilder,
+    ExceptionHandler,
     Guard,
     Method,
     ResponseHeader,
 )
+from starlite.utils import is_async_callable
 
 if TYPE_CHECKING:  # pragma: no cover
     from starlite.app import Starlite
@@ -70,6 +72,7 @@ class HTTPRouteHandler(BaseRouteHandler):
         "resolved_before_request",
         "resolved_headers",
         "resolved_response_class",
+        "resolved_exception_handlers",
         "response_class",
         "response_description",
         "response_headers",
@@ -78,6 +81,7 @@ class HTTPRouteHandler(BaseRouteHandler):
         "tags",
         "template_name",
         "sync_to_thread",
+        "exception_handlers",
     )
 
     @validate_arguments(config={"arbitrary_types_allowed": True})
@@ -97,6 +101,9 @@ class HTTPRouteHandler(BaseRouteHandler):
         status_code: Optional[int] = None,
         cache: Union[bool, int] = False,
         cache_key_builder: Optional[CacheKeyBuilder] = None,
+        exception_handlers: Optional[Dict[Union[int, Type[Exception]], ExceptionHandler]] = None,
+        # sync only
+        sync_to_thread: bool = False,
         # OpenAPI related attributes
         content_encoding: Optional[str] = None,
         content_media_type: Optional[str] = None,
@@ -108,8 +115,6 @@ class HTTPRouteHandler(BaseRouteHandler):
         response_description: Optional[str] = None,
         summary: Optional[str] = None,
         tags: Optional[List[str]] = None,
-        # sync only
-        sync_to_thread: bool = False,
     ):
         if not http_method:
             raise ImproperlyConfiguredException("An http_method kwarg is required")
@@ -136,6 +141,10 @@ class HTTPRouteHandler(BaseRouteHandler):
         self.media_type = media_type
         self.response_class = response_class
         self.response_headers = response_headers
+        self.exception_handlers = exception_handlers
+        self.cache = cache
+        self.cache_key_builder = cache_key_builder
+        self.sync_to_thread = sync_to_thread
         # OpenAPI related attributes
         self.content_encoding = content_encoding
         self.content_media_type = content_media_type
@@ -156,9 +165,9 @@ class HTTPRouteHandler(BaseRouteHandler):
         self.resolved_before_request: Union[
             Optional[BeforeRequestHandler], Type[BaseRouteHandler.empty]
         ] = BaseRouteHandler.empty
-        self.cache = cache
-        self.cache_key_builder = cache_key_builder
-        self.sync_to_thread = sync_to_thread
+        self.resolved_exception_handlers: Union[
+            Dict[Union[int, Type[Exception]], ExceptionHandler], Type[BaseRouteHandler.empty]
+        ] = BaseRouteHandler.empty
 
     def __call__(self, fn: AnyCallable) -> "HTTPRouteHandler":
         """
@@ -179,7 +188,11 @@ class HTTPRouteHandler(BaseRouteHandler):
         )
 
     def resolve_response_class(self) -> Type[Response]:
-        """Return the closest custom Response class in the owner graph or the default Response class"""
+        """
+        Returns the closest custom Response class in the owner graph or the default Response class.
+
+        This method is memoized so the computation occurs only once.
+        """
         if self.resolved_response_class is BaseRouteHandler.empty:
             self.resolved_response_class = Response
             for layer in self.ownership_layers():
@@ -191,22 +204,22 @@ class HTTPRouteHandler(BaseRouteHandler):
     def resolve_response_headers(self) -> Dict[str, ResponseHeader]:
         """
         Returns all header parameters in the scope of the handler function
+
+        This method is memoized so the computation occurs only once.
         """
         if self.resolved_headers is BaseRouteHandler.empty:
             headers: Dict[str, ResponseHeader] = {}
-            for layer in self.ownership_layers():
-                for key, value in (layer.response_headers or {}).items():
-                    if key not in headers:
-                        headers[key] = value
+            for layer in reversed(list(self.ownership_layers())):
+                headers = {**headers, **(layer.response_headers or {})}
             self.resolved_headers = headers
         return cast(Dict[str, ResponseHeader], self.resolved_headers)
 
     def resolve_before_request(self) -> Optional[BeforeRequestHandler]:
         """
-        Resolves the before_handler handler by starting from the handler and moving up.
+        Resolves the before_handler handler by starting from the route handler and moving up.
 
         If a handler is found it is returned, otherwise None is set.
-        This mehtod is memoized so the computation occurs only once
+        This method is memoized so the computation occurs only once
         """
         if self.resolved_before_request is BaseRouteHandler.empty:
             for layer in self.ownership_layers():
@@ -222,10 +235,10 @@ class HTTPRouteHandler(BaseRouteHandler):
 
     def resolve_after_request(self) -> Optional[AfterRequestHandler]:
         """
-        Resolves the after_request handler by starting from the handler and moving up.
+        Resolves the after_request handler by starting from the route handler and moving up.
 
         If a handler is found it is returned, otherwise None is set.
-        This mehtod is memoized so the computation occurs only once
+        This method is memoized so the computation occurs only once
         """
         if self.resolved_after_request is BaseRouteHandler.empty:
             for layer in self.ownership_layers():
@@ -239,6 +252,19 @@ class HTTPRouteHandler(BaseRouteHandler):
                 self.resolved_after_request = self.resolved_after_request.__func__
         return cast(Optional[AfterRequestHandler], self.resolved_after_request)
 
+    def resolve_exception_handlers(self) -> Dict[Union[int, Type[Exception]], ExceptionHandler]:
+        """
+        Resolves the exception_handlers by starting from the route handler and moving up.
+
+        This method is memoized so the computation occurs only once.
+        """
+        if self.resolved_exception_handlers is BaseRouteHandler.empty:
+            exception_handlers: Dict[Union[int, Type[Exception]], ExceptionHandler] = {}
+            for layer in reversed(list(self.ownership_layers())):
+                exception_handlers = {**exception_handlers, **(layer.exception_handlers or {})}
+            self.resolved_exception_handlers = exception_handlers
+        return cast(Dict[Union[int, Type[Exception]], ExceptionHandler], self.resolved_exception_handlers)
+
     @property
     def http_methods(self) -> List[Method]:
         """
@@ -320,7 +346,7 @@ class HTTPRouteHandler(BaseRouteHandler):
             )
         # run the after_request hook handler
         if after_request:
-            if iscoroutinefunction(after_request):
+            if is_async_callable(after_request):
                 response = await after_request(response)  # type: ignore
             else:
                 response = await run_sync(after_request, response)  # type: ignore
@@ -346,6 +372,9 @@ class get(HTTPRouteHandler):
         status_code: Optional[int] = None,
         cache: Union[bool, int] = False,
         cache_key_builder: Optional[CacheKeyBuilder] = None,
+        exception_handlers: Optional[Dict[Union[int, Type[Exception]], ExceptionHandler]] = None,
+        # sync only
+        sync_to_thread: bool = False,
         # OpenAPI related attributes
         content_encoding: Optional[str] = None,
         content_media_type: Optional[str] = None,
@@ -357,8 +386,6 @@ class get(HTTPRouteHandler):
         response_description: Optional[str] = None,
         summary: Optional[str] = None,
         tags: Optional[List[str]] = None,
-        # sync only
-        sync_to_thread: bool = False,
     ):
         super().__init__(
             http_method=HttpMethod.GET,
@@ -385,6 +412,7 @@ class get(HTTPRouteHandler):
             summary=summary,
             tags=tags,
             sync_to_thread=sync_to_thread,
+            exception_handlers=exception_handlers,
         )
 
 
@@ -404,6 +432,9 @@ class post(HTTPRouteHandler):
         status_code: Optional[int] = None,
         cache: Union[bool, int] = False,
         cache_key_builder: Optional[CacheKeyBuilder] = None,
+        exception_handlers: Optional[Dict[Union[int, Type[Exception]], ExceptionHandler]] = None,
+        # sync only
+        sync_to_thread: bool = False,
         # OpenAPI related attributes
         content_encoding: Optional[str] = None,
         content_media_type: Optional[str] = None,
@@ -415,8 +446,6 @@ class post(HTTPRouteHandler):
         response_description: Optional[str] = None,
         summary: Optional[str] = None,
         tags: Optional[List[str]] = None,
-        # sync only
-        sync_to_thread: bool = False,
     ):
         super().__init__(
             http_method=HttpMethod.POST,
@@ -443,6 +472,7 @@ class post(HTTPRouteHandler):
             summary=summary,
             tags=tags,
             sync_to_thread=sync_to_thread,
+            exception_handlers=exception_handlers,
         )
 
 
@@ -462,6 +492,9 @@ class put(HTTPRouteHandler):
         status_code: Optional[int] = None,
         cache: Union[bool, int] = False,
         cache_key_builder: Optional[CacheKeyBuilder] = None,
+        exception_handlers: Optional[Dict[Union[int, Type[Exception]], ExceptionHandler]] = None,
+        # sync only
+        sync_to_thread: bool = False,
         # OpenAPI related attributes
         content_encoding: Optional[str] = None,
         content_media_type: Optional[str] = None,
@@ -473,8 +506,6 @@ class put(HTTPRouteHandler):
         response_description: Optional[str] = None,
         summary: Optional[str] = None,
         tags: Optional[List[str]] = None,
-        # sync only
-        sync_to_thread: bool = False,
     ):
         super().__init__(
             http_method=HttpMethod.PUT,
@@ -501,6 +532,7 @@ class put(HTTPRouteHandler):
             summary=summary,
             tags=tags,
             sync_to_thread=sync_to_thread,
+            exception_handlers=exception_handlers,
         )
 
 
@@ -520,6 +552,9 @@ class patch(HTTPRouteHandler):
         status_code: Optional[int] = None,
         cache: Union[bool, int] = False,
         cache_key_builder: Optional[CacheKeyBuilder] = None,
+        exception_handlers: Optional[Dict[Union[int, Type[Exception]], ExceptionHandler]] = None,
+        # sync only
+        sync_to_thread: bool = False,
         # OpenAPI related attributes
         content_encoding: Optional[str] = None,
         content_media_type: Optional[str] = None,
@@ -531,8 +566,6 @@ class patch(HTTPRouteHandler):
         response_description: Optional[str] = None,
         summary: Optional[str] = None,
         tags: Optional[List[str]] = None,
-        # sync only
-        sync_to_thread: bool = False,
     ):
         super().__init__(
             http_method=HttpMethod.PATCH,
@@ -559,6 +592,7 @@ class patch(HTTPRouteHandler):
             summary=summary,
             tags=tags,
             sync_to_thread=sync_to_thread,
+            exception_handlers=exception_handlers,
         )
 
 
@@ -578,6 +612,9 @@ class delete(HTTPRouteHandler):
         status_code: Optional[int] = None,
         cache: Union[bool, int] = False,
         cache_key_builder: Optional[CacheKeyBuilder] = None,
+        exception_handlers: Optional[Dict[Union[int, Type[Exception]], ExceptionHandler]] = None,
+        # sync only
+        sync_to_thread: bool = False,
         # OpenAPI related attributes
         content_encoding: Optional[str] = None,
         content_media_type: Optional[str] = None,
@@ -589,8 +626,6 @@ class delete(HTTPRouteHandler):
         response_description: Optional[str] = None,
         summary: Optional[str] = None,
         tags: Optional[List[str]] = None,
-        # sync only
-        sync_to_thread: bool = False,
     ):
         super().__init__(
             http_method=HttpMethod.DELETE,
@@ -617,4 +652,5 @@ class delete(HTTPRouteHandler):
             summary=summary,
             tags=tags,
             sync_to_thread=sync_to_thread,
+            exception_handlers=exception_handlers,
         )
