@@ -1,5 +1,4 @@
 from functools import partial
-from inspect import iscoroutinefunction, ismethod
 from typing import Any, Optional
 
 from anyio.to_thread import run_sync
@@ -8,10 +7,11 @@ from pydantic.typing import AnyCallable
 from typing_extensions import Type
 
 from starlite.signature import SignatureModel
+from starlite.utils import is_async_callable
 
 
 class Provide:
-    __slots__ = ("dependency", "use_cache", "value", "signature_model", "sync_to_thread")
+    __slots__ = ("dependency", "use_cache", "value", "signature_model", "sync_to_thread", "is_coroutine")
 
     def __init__(self, dependency: AnyCallable, use_cache: bool = False, sync_to_thread: bool = False):
         self.dependency = dependency
@@ -19,9 +19,7 @@ class Provide:
         self.value: Any = Undefined
         self.signature_model: Optional[Type[SignatureModel]] = None
         self.sync_to_thread = sync_to_thread
-        if ismethod(dependency) and hasattr(dependency, "__self__"):
-            # ensure that the method's self argument is preserved
-            self.dependency = partial(dependency, dependency.__self__)
+        self.is_coroutine = is_async_callable(dependency)
 
     async def __call__(self, **kwargs: Any) -> Any:
         """
@@ -31,7 +29,7 @@ class Provide:
         if self.use_cache and self.value is not Undefined:
             return self.value
         fn = partial(self.dependency, **kwargs)
-        if iscoroutinefunction(self.dependency):
+        if self.is_coroutine:
             value = await fn()
         elif self.sync_to_thread:
             value = await run_sync(fn)
