@@ -1,3 +1,4 @@
+from .helpers import is_async_callable
 from .model import convert_dataclass_to_model, create_parsed_model_field
 from .sequence import find_index, unique
 from .url import join_paths, normalize_path
@@ -7,6 +8,7 @@ __all__ = [
     "create_parsed_model_field",
     "find_index",
     "join_paths",
+    "is_async_callable",
     "normalize_path",
     "unique",
 ]
